==> brkt-cli-1.0.1.tar.gz: brkt-cli-1.0.1/brkt_cli/user_data.py <==
# Copyright 2015 Bracket Computing, Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
# https://github.com/brkt/brkt-cli/blob/master/LICENSE
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
# CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and
# limitations under the License.
import gzip
import json
import re
from email import charset
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText

import yaml

from brkt_cli.util import (
    add_token_to_brkt_config,
    get_domain_from_brkt_config
)

# The directory for files saved on the Metavisor. We require that the dest
# path for all --brkt-files be within this directory.
from StringIO import StringIO

BRKT_FILE_PREFIX = '/var/brkt/instance_config'

BRKT_CONFIG_CONTENT_TYPE = 'text/brkt-config'
BRKT_FILES_CONTENT_TYPE = 'text/brkt-files'
GUEST_FILES_CONTENT_TYPE = 'text/brkt-guest-files'

# Load the part handler.  Note that this is only used on the guest.
# On metavisor, we handle the parts in the cloudinit handler.
file_writer_handler = """
#part-handler

import os
import yaml
import pwd
import grp


def list_types():
    return ["text/brkt-files", "text/brkt-guest-files"]


def str2oct(s):
    '''Returns an integer from an octal string.'''
    return int(s, 8)


def output(msg):
    with open('/tmp/file_writer.out', 'a') as fd:
        fd.write('file_writer.handler: %s\\n' % (msg,))
    print "file_writer.handler: %s" % (msg,)


def handle_part(data, ctype, filename, payload):
    if ctype in ('__begin__', '__end__'):
        return

    file_config = yaml.safe_load(payload)

    for filename, config in file_config.iteritems():
        output('working on %s' % (filename,))
        file_dir = os.path.dirname(filename)
        if config.get('permissions'):
            file_mask = 0777 - config['permissions']
            dir_mask = 0777 - (config['permissions'] + 0111)

        # Use the current user as the owner of the file, unless they specify
        # them
        chowner = [-1, -1]
        if config.get('owner'):
            owner = config['owner']
            if isinstance(owner, str):
                owner = pwd.getpwnam(owner).pw_uid
            chowner[0] = owner
        if config.get('group'):
            group = config['group']
            if isinstance(group, str):
                group = grp.getgrnam(group).gr_gid
            chowner[1] = group

        if not os.path.exists(file_dir):
            old_umask = None
            if config.get('permissions'):
                old_umask = os.umask(dir_mask)
            output('creating directory %s' % (file_dir,))
            os.makedirs(file_dir)
            if old_umask is not None:
                os.umask(old_umask)
            os.chown(file_dir, *chowner)

        old_umask = None
        if config.get('permissions'):
            old_umask = os.umask(file_mask)
        with open(filename, 'w') as fd:
            output('writing file %s' % (filename,))
            fd.write(config.get('contents', ''))
        if old_umask is not None:
            os.umask(old_umask)
        os.chown(filename, *chowner)
"""


# Avoid base64 encoding the MIME parts
UTF8_CHARSET = charset.Charset('utf-8')
UTF8_CHARSET.body_encoding = None  # Python defaults to BASE64


def _new_mime_part(container, content_type, payload):
    # MIMEText will prepend to the content_type
    content_type = re.sub(r'^text/', '', content_type)
    message = MIMEText(payload, content_type, 'utf-8')
    del message['Content-Transfer-Encoding']
    message.set_payload(payload, UTF8_CHARSET)
    container.attach(message)


class UserDataContainer(object):
    def __init__(self):
        self.parts = []
        self.files_config = {}
        self.add_part('text/part-handler', file_writer_handler)

    def add_part(self, mimetype, content):
        self.parts.append((mimetype, content))

    def add_file(self, filename, content, content_type):
        if content_type not in self.files_config:
            self.files_config[content_type] = {}
        self.files_config[content_type][filename] = {
            'contents': content,
        }

    def add_instance_config_file(self, filename, content):
        self.add_file(filename, content, BRKT_FILES_CONTENT_TYPE)

    def to_mime_text(self):
        # These hard coded strings are to avoid having diffs in userdata
        # when nothing changed. Without this AWS sees the userdata has changed
        # (the MIME boundary or "unixfrom" changed) and it relaunches the
        # instance to give it new data.
        container = MIMEMultipart(boundary='--===============HI-20131203==--')
        container._unixfrom = 'From nobody Tue Dec  3 19:00:57 2013'
        for part in self.parts:
            _new_mime_part(container, part[0], part[1])

        if self.files_config:
            for (content_type, files) in self.files_config.iteritems():
                _new_mime_part(container, content_type, yaml.safe_dump(files))

        return str(container)


def combine_user_data(brkt_config=None, proxy_config=None, ca_cert=None,
                      jwt=None):
    """ Combine the user data dictionary with the given proxy configuration
    into the gzipped multipart MIME binary that will be sent to the
    metavisor instance.

    :param brkt_config: dictionary of Bracket config values
    :param proxy_config: proxy.yaml contents
    :param jwt: JSON Web Token (string) that metavisor uses to auth with Yeti
    :param do_gzip: boolean specifying whether output is gzipped
    :return: a gzipped multipart MIME binary
    """
    udc = UserDataContainer()

    brkt_config = brkt_config or {}
    if jwt:
        add_token_to_brkt_config(jwt, brkt_config)

    brkt_config_string = json.dumps(brkt_config)
    udc.add_part(BRKT_CONFIG_CONTENT_TYPE, brkt_config_string)

    if proxy_config:
        udc.add_file(
            '/var/brkt/ami_config/proxy.yaml',
            proxy_config,
            BRKT_FILES_CONTENT_TYPE
        )
    if ca_cert:
        domain = get_domain_from_brkt_config(brkt_config)
        ca_cert_filename = "/var/brkt/ami_config/ca_cert.pem.%s" % (domain)
        udc.add_file(
            ca_cert_filename,
            ca_cert,
            BRKT_FILES_CONTENT_TYPE
        )

    user_data_string = udc.to_mime_text()
    return user_data_string


def gzip_user_data(user_data_string):
    out = StringIO()
    with gzip.GzipFile(fileobj=out, mode="w") as f:
        f.write(user_data_string)
    return out.getvalue()

==> brkt-cli-1.0.3.tar.gz: brkt-cli-1.0.3/brkt_cli/user_data.py <==
# Copyright 2015 Bracket Computing, Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
# https://github.com/brkt/brkt-cli/blob/master/LICENSE
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
# CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and
# limitations under the License.
import gzip
import re
from email import charset, message_from_string
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
from StringIO import StringIO

import yaml

# Load the part handler.  Note that this is only used on the guest.
# On metavisor, we handle the parts in the cloudinit handler.
file_writer_handler = """
#part-handler

import os
import yaml
import pwd
import grp


def list_types():
    return ["text/brkt-files", "text/brkt-guest-files"]


def str2oct(s):
    '''Returns an integer from an octal string.'''
    return int(s, 8)


def output(msg):
    with open('/tmp/file_writer.out', 'a') as fd:
        fd.write('file_writer.handler: %s\\n' % (msg,))
    print "file_writer.handler: %s" % (msg,)


def handle_part(data, ctype, filename, payload):
    if ctype in ('__begin__', '__end__'):
        return

    file_config = yaml.safe_load(payload)

    for filename, config in file_config.iteritems():
        output('working on %s' % (filename,))
        file_dir = os.path.dirname(filename)
        if config.get('permissions'):
            file_mask = 0777 - config['permissions']
            dir_mask = 0777 - (config['permissions'] + 0111)

        # Use the current user as the owner of the file, unless they specify
        # them
        chowner = [-1, -1]
        if config.get('owner'):
            owner = config['owner']
            if isinstance(owner, str):
                owner = pwd.getpwnam(owner).pw_uid
            chowner[0] = owner
        if config.get('group'):
            group = config['group']
            if isinstance(group, str):
                group = grp.getgrnam(group).gr_gid
            chowner[1] = group

        if not os.path.exists(file_dir):
            old_umask = None
            if config.get('permissions'):
                old_umask = os.umask(dir_mask)
            output('creating directory %s' % (file_dir,))
            os.makedirs(file_dir)
            if old_umask is not None:
                os.umask(old_umask)
            os.chown(file_dir, *chowner)

        old_umask = None
        if config.get('permissions'):
            old_umask = os.umask(file_mask)
        with open(filename, 'w') as fd:
            output('writing file %s' % (filename,))
            fd.write(config.get('contents', ''))
        if old_umask is not None:
            os.umask(old_umask)
        os.chown(filename, *chowner)
"""


# Avoid base64 encoding the MIME parts
UTF8_CHARSET = charset.Charset('utf-8')
UTF8_CHARSET.body_encoding = None  # Python defaults to BASE64


def _new_mime_part(container, content_type, payload):
    # MIMEText will prepend to the content_type
    content_type = re.sub(r'^text/', '', content_type)
    message = MIMEText(payload, content_type, 'utf-8')
    del message['Content-Transfer-Encoding']
    message.set_payload(payload, UTF8_CHARSET)
    container.attach(message)


def get_mime_part_payload(mime_data, part_content_type):
    """ Return the payload for the part with the specified content-type.

    Returns None if a part with the specified content-type is not found.
    """
    msg = message_from_string(mime_data)

    for part in msg.walk():
        if part.get_content_type() != part_content_type:
            continue
        return part.get_payload(decode=True)
    return None


class UserDataContainer(object):
    def __init__(self):
        self.parts = []
        self.files_config = {}
        self.add_part('text/part-handler', file_writer_handler)

    def add_part(self, mimetype, content):
        self.parts.append((mimetype, content))

    def add_file(self, filename, content, content_type):
        if content_type not in self.files_config:
            self.files_config[content_type] = {}
        self.files_config[content_type][filename] = {
            'contents': content,
        }

    def to_mime_text(self):
        # These hard coded strings are to avoid having diffs in userdata
        # when nothing changed. Without this AWS sees the userdata has changed
        # (the MIME boundary or "unixfrom" changed) and it relaunches the
        # instance to give it new data.
        container = MIMEMultipart(boundary='--===============HI-20131203==--')
        container._unixfrom = 'From nobody Tue Dec  3 19:00:57 2013'
        for part in self.parts:
            _new_mime_part(container, part[0], part[1])

        if self.files_config:
            for (content_type, files) in self.files_config.iteritems():
                _new_mime_part(container, content_type, yaml.safe_dump(files))

        return str(container)


def gzip_user_data(user_data_string):
    out = StringIO()
    with gzip.GzipFile(fileobj=out, mode="w") as f:
        f.write(user_data_string)
    return out.getvalue()
